from abc import ABC, abstractmethod


class ReprBase(ABC):
    @abstractmethod
    def __repr__(self) -> str:
        ...

    @abstractmethod
    def _repr_html_(self) -> str:
        ...

    @abstractmethod
    def _repr_markdown_(self) -> str:
        ...

    @abstractmethod
    def _repr_latex_(self) -> str:
        ...



from dataclasses import dataclass
from functools import partial
from typing import Union, List as TypingList
from collections import Mapping, Sequence

from IPython.core.display import display, HTML

from repr_utils._base import ReprBase


@dataclass
class List(ReprBase):
    values: Union[TypingList, Mapping]
    numbered: bool = False

    def __repr__(self) -> str:
        # todo improve (handle nested items)
        if isinstance(self.values, Mapping):
            return str(dict(**self.values))
        return str(list(*self.values))

    def get_value_repr(self, item) -> ReprBase:
        if isinstance(item, ReprBase):
            return item

        return List(item, self.numbered)

    def _repr_html_(self) -> str:
        values_string = None

        if isinstance(self.values, Mapping):
            values_string = "\n".join(
                [
                    f"<li>{key}: {value}</li>"
                    if not self.instance_of_any(value, [Mapping, Sequence, ReprBase])
                    else f"<li>{key}</li> {self.get_value_repr(value)._repr_html_()}"
                    for key, value in self.values.items()
                ]
            )
        else:
            values_string = "\n".join(
                [
                    f"<li>{item}</li>"
                    if not self.instance_of_any(item, [Mapping, TypingList, ReprBase])
                    else self.get_value_repr(item)._repr_html_()
                    for item in self.values
                ]
            )

        return "<{type}>\n{values}\n</{type}>".format(
            type="ol" if self.numbered else "ul", values=values_string
        )

    def _repr_markdown_(self) -> str:
        # Todo implement
        raise NotImplementedError()

    def _repr_latex_(self) -> str:
        # Todo implement
        raise NotImplementedError()

    def instance_of_any(self, obj, types: TypingList[type], false_on_string=True):
        if isinstance(obj, str):
            return False
        type_matcher = partial(isinstance, obj)
        return any(map(type_matcher, types))

    def _ipython_display_(self):
        # Todo remove if _repr_markdown_ and _repr_latex_ are implemented
        return display(HTML(self._repr_html_()))



from jinja2 import Environment, PackageLoader, Template


def get_template(file_name: str) -> Template:
    env = Environment(
        loader=PackageLoader("repr_utils", "templates"),
        keep_trailing_newline=True,
        trim_blocks=True,
        lstrip_blocks=True,
    )
    return env.get_template(file_name)



from dataclasses import dataclass
from typing import Any, List

from tabulate import tabulate

from repr_utils._base import ReprBase
from repr_utils._jinja import get_template


@dataclass
class Table(ReprBase):
    columns: List[str]
    values: List[List[str]]
    has_index: bool = False

    def __repr__(self) -> str:
        return self.__tabulate()

    def __tabulate(self, fmt: str = "github", values: List[List[Any]] = None) -> str:
        values = values or self.values
        return tabulate(values, headers=self.columns, tablefmt=fmt,)

    def _repr_markdown_(self) -> str:
        values = [[f"**{row[0]}**"] + row[1:] for row in self.values]
        return self.__tabulate(values=values)

    def _repr_latex_(self) -> str:
        return self.__tabulate(fmt="latex")

    def _repr_html_(self) -> str:
        template = get_template("table_html.jinja")
        return template.render(table=self)



from dataclasses import dataclass

from repr_utils._base import ReprBase


@dataclass
class Header(ReprBase):
    text: str
    header_number: int = 1

    def _repr_html_(self) -> str:
        return f"<h{self.header_number}>{self.text}</h{self.header_number}>"

    def _repr_markdown_(self) -> str:
        return f'{"#" * self.header_number} {self.text}'

    def _repr_latex_(self) -> str:
        return f'\\{"sub" * (self.header_number - 1)}section{{{self.text}}}'

    def __repr__(self) -> str:
        return self.text



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['repr_utils', 'repr_utils.templates']

package_data = \
{'': ['*']}

install_requires = \
['jinja2>=2.11.1,<3.0.0', 'tabulate>=0.8.7,<0.9.0']

setup_kwargs = {
    'name': 'repr-utils',
    'version': '0.1.3b1',
    'description': 'Contains common elements that when displayed will adhere to the correct format (e.g markdown, latex, html, text) in tools like jupyter notebooks',
    'long_description': '.. image:: https://github.com/luttik/repr_utils/workflows/CI/badge.svg\n    :alt: actions batch\n    :target: https://github.com/Luttik/repr_utils/actions?query=workflow%3ACI+branch%3Amaster\n.. image:: https://badge.fury.io/py/repr-utils.svg\n    :alt: pypi\n    :target: https://pypi.org/project/repr-utils/\n\n.. image:: https://codecov.io/gh/luttik/repr_utils/branch/master/graph/badge.svg\n    :alt: codecov\n    :target: https://codecov.io/gh/luttik/repr_utils\n\nREPR-UTIL\n---------\nA toolkit to quickly display elements in tools like `jupyter`_ by building upon `ipython rich display`_.\n\nContains simple objects that will automatically be converted to the right format based on the context.\n\nInstallation\n============\nRun :code:`pip install repr-utils`\n\nExamples\n========\nLook at `the examples notebook`_\n\nSupports:\n=========\n- Raw Text\n- HTML\n- Markdown\n- Latex\n\nCurrent objects:\n================\n- Header\n- Table\n\nRelevant links:\n===============\n- `pypi`_\n- `github`_\n\nContributions are appreciated.\n\n.. _`the examples notebook`: examples.ipynb\n.. _`pypi`: https://pypi.org/project/repr-utils/\n.. _`github`: https://github.com/Luttik/repr_utils/\n.. _`ipython rich display`: https://ipython.readthedocs.io/\n.. _`jupyter`: https://jupyter.org/\n',
    'author': 'luttik',
    'author_email': 'dtluttik@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Luttik/repr_utils',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
